#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from testtools import ToolsTestCase
from testroute import RouteTestCase
from testdatabase import DatabaseTestCase
from testmaps import MapsTestCase
from qmstestcase import *


class TestTest(QmsTestCase):
    def __init__(self, methodName):
        QmsTestCase.__init__(self, methodName)

    def testT(self):
        t = MapsTestCase('testLoadDem')
        t.getAppHandle()
        t.testLoadDem()
        t.testLoadMaps()


def suite():
    t = unittest.TestSuite()
    t.addTest(TestTest('testT'))
    #return t

    suiteMaps = MapsTestCase.suite()
    suiteDatbase = DatabaseTestCase.suite()
    suiteRoute = RouteTestCase.suite()
    suiteToos = ToolsTestCase.suite()

    alltests = unittest.TestSuite([suiteMaps, suiteDatbase, suiteRoute, suiteToos])
    #alltests = unittest.TestSuite([suiteMaps, suiteRoute])
    #return suiteRoute

    return alltests


unittest.TextTestRunner(verbosity=2).run(suite())



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from qmstestcase import *


class DatabaseTestCase(QmsTestCase):
    def __init__(self, methodName):
        QmsTestCase.__init__(self, methodName)


    def testConfigDatabase(self):
        self.showDaten()

        dbGroup = self.appWin().findAll('AXSplitGroup')[0].findAll('AXGroup')[0]
        # for the case an already existing db has been selected
        dbGroup.click()
        dbGroup.clickRight()
        waitShort()
        self.app.findAllR('AXMenu', '')[0].click()
        waitShort()

        dlg = self.app.find('AXWindow', 'Datenbank*')
        self.assertEqual(dlg.getText(), u'Datenbank hinzufügen...')

        # TODO configure
        #dlg.find('AXGroup').find('AXButton', 'OK').click()
        dlg.find('AXGroup').find('AXButton', 'Abbrechen').click()

    @staticmethod
    def suite():
        suite = unittest.TestSuite()

        suite.addTest(DatabaseTestCase('testConfigDatabase'))

        return suite


#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from uiautomator import *
import unittest
import os

dirCache = os.path.expanduser("~") + "/.QMapShack/"

class Direction:
    above, below, left, right = range(1, 5)


class EntryList:
    def __init__(self, app):
        self.app = app

    def grid(self, name=None): pass
    def showList(self): pass
    def focusList(self): pass

    def _isEntryActivated(self, entry, name):
        entry.click()
        self.app.sendSingleKey(DOWN)
        self.app.sendSingleKey(UP)

        self.app.sendSingleKey(RIGHT)
        waitShort()

        # check if map has not bean activated before
        g = self.grid()

        activated = False
        i = 0
        posEntry = -2
        while i < len(g):
            e = g[i]
            if e.getText() == name:
                posEntry = i
            if e.getText() == '' and posEntry == i - 1:
                    activated = True
                    break
            i += 1

        return activated


    def _findEntry(self, name):
        entry = None
        firstChar=name[0]
        # break for infinite loop
        b = 0
        while (entry is None) or (b < 15):
            b = b + 1
            self.app.sendKey(firstChar)
            grid = self.grid(name)
            if len(grid) > 0:
                entry = grid[0]
                break

        return entry

    def activateEntry(self, name, activate=True):
        self.showList()
        self.focusList()

        entry = self._findEntry(name)
        if entry is None:
            return False

        # check if map has not bean activated before
        activated = self._isEntryActivated(entry, name)
        logger.info('entry ' + name + ' is activated ' + str(activated))
        entry.click()

        if (not activate and activated) or (activate and not activated):
            logger.debug('popup 1')
            entry.selectPopupMenu(1)

        if activate and activated:
            logger.debug('popup 2')
            entry.selectPopupMenu(1)
            # map will be reorder after deactivate, thus search it again
            entry = self._findEntry(name)

            # if map has already been activated, make sure tiles will be loaded again
            logger.debug('popup 3')
            entry.selectPopupMenu(1)

        return True

class MapList(EntryList):
    def __init__(self, gui):
        EntryList.__init__(self, gui.app)
        self.gui = gui

    def grid(self, name=None):
        return self.gui.mapGrid(name)

    def showList(self):
        self.gui.showMaps()

    def focusList(self):
        self.gui.focusMapGrid()


class DemList(EntryList):
    def __init__(self, gui):
        EntryList.__init__(self, gui.app)
        self.gui = gui

    def grid(self, name=None):
        return self.gui.demGrid(name)

    def showList(self):
        self.gui.showDem()

    def focusList(self):
        self.gui.focusDemGrid()


class QmsGui(UIAutomator):
    def __init__(self):
        UIAutomator.__init__(self)


    def showRoute(self):
        self.app.findR('AXRadioButton', 'Route', False).click()
        waitShort()

    def showDaten(self):
        self.app.findR('AXRadioButton', 'Daten', False).click()
        waitShort()

    def showMaps(self):
        self.app.findR('AXRadioButton', 'Karten', False).click()
        waitShort()

    def showDem(self):
        self.app.findR('AXRadioButton', u'Dig. Höhenmodell (DEM)', False).click()
        waitShort()

    def datenProjectGrid(self, match = None):
        return self.appWin().findAll('AXSplitGroup')[0].findAll('AXGroup')[1].findAll('AXGroup', match)

    def datenDbGrid(self, match = None):
        return self.appWin().findAll('AXSplitGroup')[0].findAll('AXGroup')[0].findAll('AXGroup', match)

    def _findNeighbour(self, elements, beside, direction=Direction.above):
        x, y = beside.ax.AXPosition
        w, h = beside.ax.AXSize
        threshold = 10
        for e in elements:
            if e.valid() and e.visible():
                gx, gy = e.ax.AXPosition
                gw, gh = e.ax.AXSize
                # TODO add left / right aspect
                if direction == Direction.above:
                    if (gh + gy + threshold > y) and (gh + gy - threshold < y):
                        return e
                if direction == Direction.below:
                    if (gy + threshold > y + h) and (gy - threshold < y + h):
                        return e


    def _anyElementOf(self, elements, types):
        if not isinstance(elements, list):
            elements = [elements]
        if not isinstance(types, list):
            types = [types]

        for element in elements:
            if element.ax.AXRole in types:
                return True
        return False

    def mapGrid(self, match = None):
        mapButton = self.app.findR('AXRadioButton', 'Karten', False)
        groups = self.appWin().findAll('AXGroup', '', False)
        group = self._findNeighbour(groups, mapButton)

        return group.findAll('AXGroup')[0].findAll('AXGroup', match)
        #return self.appWin().findAll('AXGroup', '')[1].findAll('AXGroup')[0].findAll('AXGroup', match)

    def focusMapGrid(self):
        self.mapGrid()[0].parent().click()
        #self.appWin().findAll('AXGroup', '')[1].findAll('AXGroup')[0].click()

    def demGrid(self, match = None):
        demButton = self.app.findR('AXRadioButton', u'Dig. Höhenmodell (DEM)', False)
        groups = self.appWin().findAll('AXGroup', '', False)
        group = self._findNeighbour(groups, demButton)

        return group.find('AXGroup').findAll('AXGroup', match)

    def focusDemGrid(self):
        self.demGrid()[0].parent().click()

    def mapArea(self):
        groups = self.appWin(True).findAll('AXGroup', '', True)
        mapGroup = None
        for group in groups:
            if group.valid() and group.visible():
                children = group.children()
                if len(children) == 0 or not self._anyElementOf(children, ['AXGroup', 'AXSplitGroup', 'AXPopUpButton']):
                    mapGroup = group
                    break

        #UIAutomator.debug(mapGroup, 1)
        return mapGroup

    def selectMap(self, name, activate=True):
        return MapList(self).activateEntry(name, activate)

    def selectDem(self, name, activate=True):
        return DemList(self).activateEntry(name, activate)



class QmsTestCase(QmsGui, unittest.TestCase):

    def __init__(self, methodName):
        QmsGui.__init__(self)
        unittest.TestCase.__init__(self, methodName)

    def _cleanDir(self, dirPath):
        try:
            if os.path.isdir(dirPath):
                fileList = os.listdir(dirPath)
                [ os.remove(os.path.abspath(os.path.join(dirPath,fileName))) for fileName in fileList ]
        except OSError:
            pass

    def _cleanFiles(self):
        try:
            os.remove(config.get('directory', 'out.test.dir') + "/" + config.get('directory', 'vrt.out.file'))
        except OSError:
            pass
        self._cleanDir(dirCache+'WorldSat')
        self._cleanDir(dirCache+'OSM')
        self._cleanDir(dirCache+'OSM_Topo')
        self._cleanDir(config.get('directory', 'out.test.dir'))


    def setUp(self):
        self._cleanFiles()
        res = self.getAppHandle()
        self.assertTrue(res)

        self._cleanFiles()

    def tearDown(self):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from qmstestcase import *


class ToolsTestCase(QmsTestCase):
    def __init__(self, methodName):
        QmsTestCase.__init__(self, methodName)

    def testVrtBuilder(self):
        self.app.selectMenuItem('Werkzeug', 'VRT Builder')

        group = self.app.findR('AXStaticText', 'Quelldateien*').parent()

        group.findAll('AXButton', '...')[0].click()
        res = self.openFileDlg(config.get('directory', 'dem.dir.ch'))
        self.assertTrue(res)

        group.findAll('AXButton', '...')[1].click()

        res = self.safeFileDlg(config.get('directory', 'out.test.dir') , config.get('directory', 'vrt.out.file'))
        self.assertTrue(res)

        group.find('AXButton', 'Start').click()
        wait(1.0)

        group = self.app.findR('AXStaticText', 'Quelldateien*').parent()

        # test result
        resultField = group.find('AXStaticText', '')
        resultField.click()
        self.app.sendKey('a', [COMMAND])

        self.assertRegexpMatches(resultField.ax.AXSelectedText, '0...10...20...30...40...50...60...70...80...90...100 - done')
        self.assertTrue(os.path.isfile(config.get('directory', 'out.test.dir') + '/' + config.get('directory', 'vrt.out.file')))


    def testRoutinoDatabaseBuilder(self):
        self.app.selectMenuItem('Werkzeug', 'Routino Datenbank erstellen')

        group = self.app.findR('AXStaticText', 'Quelldateien*').parent()

        group.findAll('AXButton', '...')[0].click()

        res = self.openFileDlg(config.get('directory', 'pbf.dir.ch'), config.get('directory', 'pdf.file.ch'))
        self.assertTrue(res)

        group.findAll('AXButton', '...')[1].click()
        res = self.safeDirectoryDlg(config.get('directory', 'out.test.dir'))
        self.assertTrue(res)

        # FIXME here is a bug in the GUI
        group.find('AXStaticText', '').click()
        self.app.sendSingleKey(TAB)
        self.app.sendKey(config.get('directory', 'routing.out.file'))
        waitShort()

        group.find('AXButton', 'Starten').click()

        i = 1
        text = None
        while i < 15:
            i = i + 1
            wait(10)

            group = self.app.findR('AXStaticText', 'Quelldateien*').parent()

            # test result
            resultField = group.find('AXStaticText', '')
            resultField.click()
            self.app.sendKey('a', [COMMAND])
            text = resultField.ax.AXSelectedText
            if text.endswith('!!! erledigt !!!'): break

        self.assertRegexpMatches(text, '!!! erledigt !!!')
        file = config.get('directory', 'out.test.dir') + '/' + config.get('directory', 'routing.out.file')+ "-nodes.mem"
        self.assertTrue(os.path.isfile(file), file + " not fould")


    @staticmethod
    def suite():
        suite = unittest.TestSuite()

        suite.addTest(ToolsTestCase('testVrtBuilder'))
        suite.addTest(ToolsTestCase('testRoutinoDatabaseBuilder'))

        return suite


#!/usr/bin/env python
# vim:fileencoding=utf-8

from __future__ import print_function
import csv


type_csv_filename = './Profile_Types.csv'
messages_csv_filename = './Profile_Messages.csv'

enum_filename = 'gen-fit_enums.h'
enum_profiles_filename = 'gen-fit_fields.h'
profile_code_filename = 'gen-CFitProfileLockup.cpp'
profile_init_filename = 'gen-CFitProfileLockup-call.cpp'


def titlecase(s):
    s = s.replace('_', ' ')
    s = s.title()
    s = s.replace(' ', '')
    return s


def zero_if_null(s, i = None):
    return default_if_null('0', s, i)


def empty_if_none(s, i = None):
    return default_if_null('', s, i)


def default_if_null(default, s, i = None):
    if i is not None:
        if len(s) > i:
            s = s[i]
        else:
            s = ''

    if not s:
        return default
    return s


def write_enum_type(enum_file, enum_list, type_name):
    if len(enum_list) > 0 and type_name:
        count = 0
        print("typedef enum {", file=enum_file)
        for e in enum_list:
            if count + 1 < len(enum_list):
                print('    ' + e + ',', file=enum_file)
            else:
                print('    ' + e, file=enum_file)
            count = count + 1
        print('} ' + type_name + '_e;', file=enum_file)
        print(' ', file=enum_file)

def write_enum_file():
    # field enums
    count = 0
    last_type_name = ''
    enum_list = []

    with open(type_csv_filename, 'r') as csvfile:
        typereader = csv.reader(csvfile, delimiter=';', quotechar='"')
        with open(enum_filename, 'w') as enum_file:
            for row in typereader:
                type_name = row[0]
                base_type = row[1]
                value_name = row[2]
                value = row[3]
                comment = row[4]

                if count > 0:
                    if type_name:
                        write_enum_type(enum_file, enum_list, last_type_name)
                        enum_list = []
                        last_type_name = type_name

                    if value_name and value:
                        enum_list.append('e' + titlecase(last_type_name) + titlecase(value_name) + ' = ' + value)

                count = count + 1
            write_enum_type(enum_file, enum_list, last_type_name)


def write_message_enums():
    count = 0
    last_type_name = ''
    enum_list = []

    with open(messages_csv_filename, 'r') as csvfile:
        typereader = csv.reader(csvfile, delimiter=';', quotechar='"')
        with open(enum_profiles_filename, 'w') as enum_file:
            for row in typereader:
                message_name = row[0]
                field_def_nr = row[1]
                field_name = row[2]

                if count > 0:
                    if message_name:
                        # '} message_' + + '_e;'
                        write_enum_type(enum_file, enum_list, 'message_' + last_type_name)
                        enum_list = []
                        last_type_name = message_name

                    if field_name and field_def_nr:
                        enum_list.append('e' + titlecase(last_type_name) + titlecase(field_name) + ' = ' + field_def_nr)

                count = count + 1
            write_enum_type(enum_file, enum_list, 'message_' + last_type_name)



def write_profile_type(profile_file, field_list, msg_name):
    if len(field_list) > 0 and msg_name:
        print('void init' + titlecase(msg_name) + '(QMap<quint16, CFitProfile*>& profiles)', file=profile_file)
        print('{', file=profile_file)
        print('    CFitProfile* f = new CFitProfile("' + msg_name + '", eMesgNum' + titlecase(msg_name) + ');', file=profile_file)
        for f in field_list:
            print('    f->' + f + ';', file=profile_file)
        print('    profiles.insert(eMesgNum' + titlecase(msg_name) + ', f);', file=profile_file)
        print('}', file=profile_file)
        print(' ', file=profile_file)


def write_profile_init(profile_file, msg_name):
    if msg_name:
        print('    init' + titlecase(msg_name) + '(allProfiles);', file=profile_file)


base_type_map = {
    'uint8' : 'fitUint8Type', 'uint16': 'fitUint16Type', 'uint32':'fitUint32Type', 'uint64':'fitUint64Type',
    'sint8':'fitSint8Type', 'sint16':'fitSint16Type', 'sint32': 'fitSint32Type', 'sint64':'fitSint64Type',
    'uint8z':'fitUint8zType', 'uint16z':'fitUint16zType', 'uint32z':'fitUint32zType', 'uint64z':'fitUint64zType',
    'float32':'fitFloat32Type', 'float64':'fitFloat64Type',
    'string':'fitStringType', 'byte':'fitByteType', 'bool':'fitEnumType', '':'fitEnumType'}


def base_type(type):
    if type in base_type_map:
        return base_type_map[type]
    return 'fitEnumType'


def write_profiles():
    count = 0
    subcount = 0
    last_msg_name = ''
    field_names = {}
    add_list = []

    profile_file = open(profile_code_filename, 'w')
    profile_init_file = open(profile_init_filename, 'w')

    with open(messages_csv_filename, 'r') as csvfile:
        typereader = csv.reader(csvfile, delimiter=';', quotechar='"')
        for row in typereader:
            message_name = row[0]
            field_def_nr = row[1]
            field_name = row[2]
            field_type = row[3]
            array = row[4]
            components = row[5]
            scale = row[6]
            offset = row[7]
            units = row[8]
            bits = row[9]
            accumulate = row[10]
            ref_field_name = row[11]
            ref_field_value = row[12]

            if count > 0:
                if message_name:
                    # new message
                    write_profile_type(profile_file, add_list, last_msg_name)
                    write_profile_init(profile_init_file, last_msg_name)
                    add_list = []
                    last_msg_name = message_name

                elif field_name:
                    type_name = base_type(field_type)
                    field_names[field_name] = field_type

                    if field_def_nr and not components:
                        # normal type
                        scale = zero_if_null(scale)
                        offset = zero_if_null(offset)

                        last_field_def_nr = 'e' + titlecase(last_msg_name) + titlecase(field_name)
                        add_list.append('addField("' + field_name + '", ' + type_name + ', ' + last_field_def_nr + ', '
                                        + scale + ', ' + offset + ', "' + units + '")')
                        subcount = 0
                        # end normal type
                    else:
                        # sub type or component type

                        # ref field name is for all elements the same, so take the first one
                        ref_field_name = empty_if_none(ref_field_name.split(','), 0)
                        ref_field_value_list = ref_field_value.split(',')
                        components_list = components.split(',')
                        scale_list = scale.split(',')
                        offset_list = offset.split(',')
                        units_list = units.split(',')
                        bits_list = bits.split(',')

                        if not field_def_nr:
                            # sub type
                            enum_field = 'e' + titlecase(last_msg_name) + titlecase(ref_field_name)

                            # case it has components (scale, offset and units are undefined)
                            scale = '0'
                            offset = '0'
                            units = ''
                            i = 0
                            for a in ref_field_value_list:
                                if not components:
                                    scale = zero_if_null(scale_list, i)
                                    offset = zero_if_null(offset_list, i)
                                    units = empty_if_none(units_list, i)

                                if ref_field_name not in field_names or not field_names[ref_field_name]:
                                    enum_field_value = 'e' + titlecase(ref_field_name) + titlecase(ref_field_value_list[i])
                                else:
                                    enum_field_value = 'e' +titlecase(field_names[ref_field_name]) + titlecase(ref_field_value_list[i])

                                add_list.append('addSubfield("' + field_name + '", ' + type_name + ', ' + last_field_def_nr + ', '
                                                + scale + ', ' + offset + ', "' + units + '", ' + enum_field + ', ' + enum_field_value+')')
                                i = i + 1
                                subcount = subcount + 1
                            #  end sub type
                        if components:
                            # component type
                            if field_def_nr:
                                # a component needs always a parent field. add one
                                last_field_def_nr = 'e' + titlecase(last_msg_name) + titlecase(field_name)
                                add_list.append('addField("' + field_name + '", ' + type_name + ', ' + last_field_def_nr + ', 0, 0, "")')
                                subaccess_field = ''
                            else:
                                # parent field for component is sub field
                                subaccess_field = str(subcount - 1) + ', '

                            i = 0
                            for a in components_list:
                                scale = zero_if_null(scale_list, i)
                                if not scale:
                                    scale = zero_if_null(scale_list, 0)
                                offset = zero_if_null(offset_list, i)
                                units = empty_if_none(units_list, i)
                                if not units:
                                    units = empty_if_none(units_list, 0)
                                bits = zero_if_null(bits_list, i)
                                if not bits:
                                    bits = zero_if_null(bits_list, 0)
                                enum_component = 'e' + titlecase(last_msg_name) + titlecase(components_list[i])
                                add_list.append('addComponent('+ subaccess_field + '"' + field_name + '", ' + type_name + ', ' +
                                                last_field_def_nr + ', ' + scale + ', ' + offset + ', "' + units + '", ' +
                                                enum_component + ', ' + bits + ')')
                                i = i + 1
                            # end component type
            count = count + 1
        write_profile_type(profile_file, add_list, last_msg_name)
        write_profile_init(profile_init_file, last_msg_name)


write_enum_file()
write_message_enums()
write_profiles()


#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from qmstestcase import *


class RouteTestCase(QmsTestCase):
    def __init__(self, methodName):
        QmsTestCase.__init__(self, methodName)


    def testConfigRoutino(self):
        self.showRoute()

        # first dropdown Routino (offline)
        res = self.appWin().find('AXPopUpButton').selectPopupElement('Routino (offline)')
        self.assertTrue(res, 'set routino popup button failed')

        routeButton = self.app.findAllR('AXRadioButton', 'Route')[0]
        groups = self.appWin().findAll('AXGroup', '')
        group = self._findNeighbour(groups, routeButton)

        # setup database
        group.find('AXButton', '...').click()
        dlgRoutinoDb = self.app.find('AXWindow')

        pathes = dlgRoutinoDb.find('AXList').findAll('AXStaticText')
        routinoPath = config.get('directory', 'mem.dir.ch')
        routinoPathAdded = False
        for path in pathes:
            if path.getText() == routinoPath:
                routinoPathAdded = True
                logger.info('routino path ' + routinoPath + ' already added')
                break

        if routinoPathAdded == False:
            logger.info('adding routino path ' + routinoPath)
            dlgRoutinoDb.find('AXButton', '...').click()
            res = self.safeDirectoryDlg(routinoPath)
            self.assertTrue(res, 'set directory dialog failed')

        dlgRoutinoDb.find('AXGroup').find('AXButton', 'OK').click()

        # the group for the Routino Route settings
        dropdowns = group.findAll('AXPopUpButton')
        for dropdown in dropdowns:
            dropdown.selectPopupElement('Fahrrad')
            dropdown.selectPopupElement('Deutsch')
            dropdown.selectPopupElement('Schnellste')
            dropdown.selectPopupElement('Europa')

            dropdown.selectPopupElement('Switzerland')


    def testSearchGoogle(self):
        # --> search google
        self.showDaten()
        town = 'Grindelwald'

        grid = self.datenProjectGrid()
        if len(grid) < 2 or grid[1].getText() <> '':
            # click menu
            self.app.selectMenuItem('Projekt', 'Mit Google suchen')
            googleInput = self.datenProjectGrid()[1]
        else:
            googleInput = grid[1]

        googleInput.click()
        waitShort()

        self.app.sendKey("a", [COMMAND])
        self.app.sendSingleKey(BACKSPACE)

        self.app.sendKey(town)
        self.app.sendSingleKey(RETURN)
        #self.app.sendKey('\n')
        waitShort()

        resultTown = self.datenProjectGrid()[3]
        self.assertRegexpMatches(resultTown.getText(), town)
        resultTown.doubleClick()
        waitShort()

        map = self.mapArea()
        map.click()
        waitShort()

        # --> position in status bar
        coordField = self.appWin().find('AXStaticText').find('AXStaticText')
        logger.debug(unicode(coordField.getText()).encode('utf8'))
        self.assertRegexpMatches(coordField.getText(), u'N46\xb0 37.\d{3} E008\xb0 02.\d{3}')
        # \xb0 °
        map.click()


    def testRoute(self):
        # relies on testSearchGoogle
        #self.startApp()
        self.showMaps()

        # --> routing with routino
        # click to center of map, where the searched place is
        map = self.mapArea()
        map.click()
        waitShort()

        # auto routing
        self.app.sendKey('a', [CONTROL])
        waitShort()

        map.selectPopupMenu(2)
        waitShort()

        map.moveAndClick(10, 112)
        waitShort()
        map.click()
        waitShort()
        map.clickRight()

        text = map.find('AXStaticText', 'Track*')
        self.assertTrue(text.valid())
        logger.debug(unicode(text.getText()).encode('utf8'))
        self.assertRegexpMatches(text.getText(), "35\d m")


    def testSaveRoute(self):
        projName='uitest'

        # --> save route
        map = self.mapArea()
        map.find('AXButton', 'Als neu speichern').click()
        #map.find('AXButton', 'Abbrechen').click()

        projDlg = self.app.find('AXWindow', u'Ein Projekt auswählen...')
        self.assertTrue(projDlg.valid())
        # FIXME here is a bug in the GUI
        #projDlg.find('AXList').findAll('AXStaticText')[0].click()
        #projDlg.find('AXTextField').click()
        self.app.sendSingleKey(TAB)
        self.app.sendKey('uitest')

        # *.qms, *.gpx, Datenbank
        projDlg.find('AXRadioButton', '*.gpx').click()
        projDlg.find('AXGroup').find('AXButton', 'OK').click()

        nameDlg = self.app.find('AXWindow', u'Name bearbeiten...')
        self.assertTrue(nameDlg.valid())
        # FIXME here is a bug in the GUI
        #nameDlg.find('AXTextField').click()
        self.app.sendKey(projName)
        nameDlg.find('AXGroup').find('AXButton', 'OK').click()
        waitShort()

        self.showDaten()

        # somehow the app reference has been lost, get it again
        self.getAppHandle()

        grid = self.datenProjectGrid(projName)
        self.assertIs(len(grid), 1)
        grid[0].doubleClick()
        grid = self.datenProjectGrid(projName)
        self.assertIs(len(grid), 2)
        grid[1].clickRight()
        grid[1].selectPopupMenu(1)

        buttonView = self.appWin().findR('AXRadioButton', projName)
        groups = self.appWin().findAll('AXGroup', '')
        detailView = self._findNeighbour(groups, buttonView, Direction.below).find('AXSplitGroup')

        detailView.findAll('AXGroup')[1].find('AXRadioButton', 'Punkte').click()
        table = detailView.findAll('AXGroup')[0].find('AXGroup').findAll('AXGroup')

        # 10 columns per row
        self.assertEqual(table[2].getText(), '1045 m')

        # the next lines are for a route
        if False:
            trkDlg = self.app.find('AXWindow', projName)
            self.assertTrue(trkDlg.valid())
            self.assertEqual(trkDlg.find('AXGroup').find('AXTextField').getText(), projName)

            # close dialog
            trkDlg.find('AXButton').click()

        # close track
        grid[0].selectPopupMenu(8)
        waitShort()

        questionDlg = self.app.find('AXWindow', '')
        questionDlg.find('AXGroup').find('AXButton', 'Nein').click()


    def testEditRoute(self):
        # TODO
        pass


    @staticmethod
    def suite():
        suite = unittest.TestSuite()

        suite.addTest(RouteTestCase('testConfigRoutino'))
        suite.addTest(RouteTestCase('testSearchGoogle'))
        suite.addTest(RouteTestCase('testRoute'))
        suite.addTest(RouteTestCase('testSaveRoute'))
        suite.addTest(RouteTestCase('testEditRoute'))

        return suite


#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""

from qmstestcase import *

import datetime


class MapsTestCase(QmsTestCase):
    def __init__(self, methodName):
        QmsTestCase.__init__(self, methodName)

    def testLoadDem(self):
        dirDem = config.get('directory', 'dem.dir')

        self.app.selectMenuItem('Datei', 'DEM Verzeichnisse angeben')

        demPathDlg = self.app.find('AXWindow', u'Pfad für DEM Dateien setzen')

        pathes = demPathDlg.find('AXList').findAll('AXStaticText')
        mapThere = False
        for path in pathes:
            if path.getText() == dirDem:
                mapThere = True

        if mapThere == False:
            demPathDlg.findAll('AXButton', '...')[0].click()
            waitShort()
            self.safeDirectoryDlg(dirDem)
            waitShort()

        self.assertTrue(demPathDlg.find('AXList').find('AXStaticText', dirDem).valid())

        demPathDlg.find('AXGroup').find('AXButton', 'OK').click()
        waitShort()

        self.getAppHandle()

        # activate map
        demName = config.get('directory', 'dem.name')
        res = self.selectDem(demName, True)
        self.assertTrue(res)



    def testLoadMaps(self):
        mapDir = config.get('directory', 'map.dir')

        self.app.selectMenuItem('Datei', 'Kartenverzeichnisse angeben')

        mapPathDlg = self.app.find('AXWindow', 'Kartenpfad einrichten')

        pathes = mapPathDlg.find('AXList').findAll('AXStaticText')
        mapThere = False
        for path in pathes:
            if path.getText() == mapDir:
                mapThere = True

        if mapThere == False:
            mapPathDlg.findAll('AXButton', '...')[1].click()
            self.safeDirectoryDlg(mapDir)
            waitShort()

        self.assertTrue(mapPathDlg.find('AXList').find('AXStaticText', mapDir).valid())

        mapPathDlg.find('AXGroup').find('AXButton', 'OK').click()
        waitShort()

        self.getAppHandle()

        # activate map
        mapName = config.get('directory', 'map.name')
        res = self.selectMap(mapName, True)
        self.assertTrue(res)


    def testOnlineMap(self):
        type = 'OSM'
        self.selectMap('OSM', True)

        # check downloaded tiles
        dateCreated = datetime.datetime.now() - datetime.timedelta(minutes=1)

        dirPath = dirCache+type
        tiles = 0

        wait(3.0)
        fileList = os.listdir(dirPath)
        for fileName in fileList:
            t = os.path.getmtime(os.path.abspath(os.path.join(dirPath,fileName)))
            dateFile = datetime.datetime.fromtimestamp(t)
            if dateCreated < dateFile:
                tiles = tiles + 1
        self.assertTrue(tiles > 8, 'only %s tiles found' % tiles)


    @staticmethod
    def suite():
        suite = unittest.TestSuite()

        suite.addTest(MapsTestCase('testLoadDem'))
        suite.addTest(MapsTestCase('testLoadMaps'))
        suite.addTest(MapsTestCase('testOnlineMap'))

        return suite



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
/**********************************************************************************************
Copyright (C) 2016 Ivo Kronenberg

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************************************/
"""


import atomac
import atomac.AXKeyboard as AXKeyboard

from collections import deque
import time
import logging
import ConfigParser


def initConfig():
    config = ConfigParser.ConfigParser()
    config.read('uitest.cfg')
    return config


def initLogger():
    FORMAT = '%(asctime)-15s %(message)s'
    logging.basicConfig(format=FORMAT)
    log = logging.getLogger('uitest')
    log.setLevel(logging.INFO)
    return log


logger = initLogger()
config = initConfig()


def waitShort():
    time.sleep(config.getfloat('flow', 'wait'))

def wait(sec):
    time.sleep(sec)



class UIBase:
    def __init__(self, ax, app):
        self.ax = ax
        self.cache = {}
        if app is None:
            self.app = self
        else:
            self.app = app
        if ax is None:
            raise ReferenceError()

    def _lookup(self, find, type, match, clear):
        key = find + '#' + type
        if match is not None:
            key = key + '#' + match

        logger.debug(unicode(key).encode('utf8'))
        if key in self.cache:
            if clear:
                del self.cache[key]
                return None
            logger.debug('match')
            return self.cache[key]
        logger.debug('no match')
        return None

    def _put(self, element, find, type, match):
        key = find + '#' + type
        if match is not None:
            key = key + '#' + match

        self.cache[key] = element
        return element


    def centreCoordinates(self):
        x, y = self.ax.AXPosition
        width, height = self.ax.AXSize
        return (x + width / 2, y + height / 2)

    def valid(self):
        return self.ax is not None

    def enabled(self):
        return self.valid() and self.ax.AXEnabled

    def _measureEnter(self, method):
        self.start = time.time()
        self.methodname = method

    def _measureLeave(self):
        end = time.time()
        logger.debug("exec time " + self.methodname + " " + str(end - self.start))


    def find(self, type, textmatch=None, refresh=True):
        self._measureEnter("find")
        try:
            v = self._lookup('find', type, textmatch, refresh)
            if v is not None: return v

            if textmatch is not None:
                e = UIElement(self.ax.findFirst(AXRole=type, AXTitle=textmatch), self.app)
                return self._put(e, 'find', type, textmatch)
            else:
                e = UIElement(self.ax.findFirst(AXRole=type), self.app)
                return self._put(e, 'find', type, textmatch)
        finally:
            self._measureLeave()

    def findNr(self, type, textmatch, nr, refresh=True):
        return self.findAll(type, textmatch, refresh)[nr]

    def findAll(self, type, textmatch=None, refresh=True):
        self._measureEnter("findAll")
        try:
            v = self._lookup('findAll', type, textmatch, refresh)
            if v is not None: return v

            if textmatch is not None:
                axs = self.ax.findAll(AXRole=type, AXTitle=textmatch)
            else:
                axs = self.ax.findAll(AXRole=type)
            e = []
            if axs is not None and len(axs) > 0:
                for ax in axs:
                    e.append(UIElement(ax, self.app))

            return self._put(e, 'findAll', type, textmatch)
        finally:
            self._measureLeave()


    def findAllR(self, type, textmatch=None, refresh=True):
        self._measureEnter("findAllR")
        try:
            v = self._lookup('findAllR', type, textmatch, refresh)
            if v is not None: return v

            if textmatch is not None:
                axs = self.ax.findAllR(AXRole=type, AXTitle=textmatch)
            else:
                axs = self.ax.findAllR(AXRole=type)
            e = []
            if axs is not None and len(axs) > 0:
                for ax in axs:
                    e.append(UIElement(ax, self.app))

            return self._put(e, 'findAllR', type, textmatch)
        finally:
            self._measureLeave()

    def findR(self, type, textmatch=None, refresh=True):
        self._measureEnter("findR")
        try:
            v = self._lookup('findR', type, textmatch, refresh)
            if v is not None: return v

            if textmatch is not None:
                e = UIElement(self.ax.findFirstR(AXRole=type, AXTitle=textmatch), self.app)
                return self._put(e, 'findR', type, textmatch)
            else:
                e = UIElement(self.ax.findFirstR(AXRole=type), self.app)
                return self._put(e, 'findR', type, textmatch)
        finally:
            self._measureLeave()


    def parent(self):
        return UIElement(self.ax.AXParent, self.app)

    def children(self):
        e = []
        axs = self.ax.AXChildren
        if axs is not None and len(axs) > 0:
            for ax in axs:
                e.append(UIElement(ax, self.app))

        return e

    def visible(self):
        x, y = self.ax.AXPosition
        return x >= 0 and y >= 0



# Special keys
TAB            = '<tab>'
RETURN         = '<return>'
ESCAPE         = '<escape>'
CAPS_LOCK      = '<capslock>'
DELETE         = '<delete>'
NUM_LOCK       = '<num_lock>'
SCROLL_LOCK    = '<scroll_lock>'
PAUSE          = '<pause>'
BACKSPACE      = '<backspace>'
INSERT         = '<insert>'

# Cursor movement
UP             = '<cursor_up>'
DOWN           = '<cursor_down>'
LEFT           = '<cursor_left>'
RIGHT          = '<cursor_right>'
PAGE_UP        = '<page_up>'
PAGE_DOWN      = '<page_down>'
HOME           = '<home>'
END            = '<end>'

# Modifier keys
COMMAND_L      = '<command_l>'
SHIFT_L        = '<shift_l>'
OPTION_L       = '<option_l>'
CONTROL_L      = '<control_l>'

COMMAND_R      = '<command_r>'
SHIFT_R        = '<shift_r>'
OPTION_R       = '<option_r>'
CONTROL_R      = '<control_r>'

# Default modifier keys -> left:
COMMAND        = COMMAND_L
SHIFT          = SHIFT_L
OPTION         = OPTION_L
CONTROL        = CONTROL_L


class KeyboardHandler (UIBase):

    def __init__(self, ax, app):
        UIBase.__init__(self, ax, app)

    def _replace(self, str):
        return str.replace(u'ö', '\xd6')

    def sendKey(self, key, modifiers = []):
        if len(modifiers) == 0:
            self.app.ax.sendKeys(key)
        else:
            self.app.ax.sendKeyWithModifiers(key, modifiers)

    def sendSingleKey(self, key):
        self.app.ax.sendKey(key)



class MouseHandler(UIBase):

    def __init__(self, ax, app):
        UIBase.__init__(self, ax, app)
        self.dx = 0
        self.dy = 0

    def _coordinates(self):
        (x, y) = self.centreCoordinates()
        return (x+self.dx, y+self.dy)

    def click(self):
        #self.ax.Press()
        self.app.ax.clickMouseButtonLeft(self._coordinates())

    def doubleClick(self):
        self.app.ax.doubleClickMouse(self._coordinates())

    def clickRight(self):
        self.app.ax.clickMouseButtonRight(self._coordinates())

    def moveAndClick(self, dx, dy):
        self.dx = dx
        self.dy = dy
        self.app.ax.clickMouseButtonLeft(self._coordinates())


class UIElement(UIBase, MouseHandler):

    def __init__(self, ax, app):
        UIBase.__init__(self, ax, app)
        MouseHandler.__init__(self, ax, app)

    def setValue(self, value):
        self.ax.AXValue = value

    def getText(self):
        if self.ax.AXValue is not None:
            return self.ax.AXValue

        return self.ax.AXTitle

    def selectPopupElement(self, title):
        langElements = self.find('AXList').findAll('AXStaticText', title)
        if len(langElements) == 1:
            self.click()
            waitShort()
            langElements[0].click()
            return True

        return False

    def selectPopupMenu(self, nr):
        self.clickRight()
        waitShort()

        i = 0
        while i < nr:
            self.app.sendSingleKey(DOWN)
            i = i + 1
        self.app.sendSingleKey(RETURN)


class UIApp(UIElement, KeyboardHandler):

    def __init__(self, app):
        UIElement.__init__(self, app, None)
        KeyboardHandler.__init__(self, app, None)

        # bugfixes for event queue
        self.app.keyboard = AXKeyboard.loadKeyboard()
        self.app.eventList = deque()


    @staticmethod
    def getApp(bundleName):
        ax = None
        try:
            ax = atomac.getAppRefByBundleId(bundleName)
        except ValueError:
            ax = atomac.launchAppByBundleId(bundleName)
            if ax is None:
                ax = atomac.getAppRefByBundleId(bundleName)

        while ax.AXTitle is None:
            ax = atomac.getAppRefByBundleId(bundleName)

        ax.activate()
        return UIApp(ax)


    def selectMenuItem(self, menuText, menuItemText):
        # AXMenuBar -> AXMenuBarItem -> AXMenu -> AXMenuItem
        menu = self.app.ax.AXMenuBar.findFirst(AXRole='AXMenuBarItem', AXTitle=menuText)
        menuitem = menu.findFirstR(AXRole='AXMenuItem', AXTitle=menuItemText)
        menu.Press()
        menuitem.Press()


class UIAutomator:
    def __init__(self):
        self.app = None

        self.appName = 'QMapShack*'
        self.bundleId = 'org.qlandkarte.QMapShack'

    def appWin(self, reload=False):
        return self.app.find('AXWindow', None, reload)

    def openFileDlg(self, dirname, filename=None):
        fileDlg = self.app.find('AXWindow', 'Dateien*')
        if fileDlg.valid() == False:
            return False

        # input for directory
        self.app.sendKey('7', [SHIFT]) # /
        waitShort()

        dirInput = fileDlg.find('AXSheet')
        dirInput.find('AXTextField').setValue(dirname)
        waitShort()
        dirInput.findNr('AXButton', None, 1).click()

        # select file
        subelement = fileDlg.find('AXGroup').find('AXSplitGroup').find('AXSplitGroup').find('AXScrollArea').find('AXOutline')
        files = subelement.findAllR('AXTextField')

        found = False
        if filename is not None:
            for f in files:
                if f.ax.AXFilename == filename:
                    f.click()
                    break

        if found == False:
            files[0].click()

        # open
        fileDlg.find('AXButton', '\xd6ffnen').click()
        waitShort()

        return True


    def safeFileDlg(self, dirname, filename = None):
        fileDlg = self.app.find('AXWindow', 'Zieldatei*')
        if fileDlg.valid() == False:
            return False

        # input for directory
        self.app.sendKey('7', [SHIFT])
        waitShort()

        dirInput = fileDlg.find('AXSheet')
        dirInput.find('AXTextField').setValue(dirname)
        self.app.sendKey('\n')
        waitShort()

        # set filename
        fileDlg.find('AXTextField').setValue(filename)
        waitShort()

        fileDlg.find('AXButton', 'Sichern').click()

        waitShort()

        return True


    def safeDirectoryDlg(self, dirname):
        # 'Zielpfad*'
        fileDlg = self.app.find('AXWindow')
        if fileDlg.valid() == False:
            return False

        # input for directory
        self.app.sendKey('7', [SHIFT])
        waitShort()

        dirInput = fileDlg.find('AXSheet')
        dirInput.find('AXTextField').setValue(dirname)
        self.app.sendKey('\n')
        waitShort()

        fileDlg.find('AXButton', u'Öffnen').click()

        waitShort()

        return True


    def getAppHandle(self):
        self.app = UIApp.getApp(self.bundleId)
        return self.app.valid()


    def closeApp(self):
        self.app.sendKey('q', [COMMAND])
        # TODO assert
        return True


    @staticmethod
    def debug(element, depth=1, nr=0, inv=0, typeMap=None):
        if depth <= inv and depth > 0:
            return

        if element is None:
            print "None element"
        elif isinstance(element, list):
            i = 0
            ctypeMap = {}
            for e in element:
                UIAutomator.debug(e, depth, i, inv, ctypeMap)
                i = i+1
        else:
            if typeMap is None: typeMap = {}
            type = element.ax.AXRole
            tcount = 0
            if type in typeMap:
                tcount = tcount + typeMap[type]
            typeMap[type] = tcount

            pre = "".ljust(inv*4)
            print pre + str(nr) + " (" + str(typeMap[type]) +") " + str(element.ax)
            print pre +"  attributes " + str(element.ax.getAttributes())
            print pre +"  actions: " + str(element.ax.getActions())
            print pre +"  Pos / Size: " + str(element.ax.AXPosition) + " " + str(element.ax.AXSize)
            print pre +"  parent: "+ str(element.parent().ax)
            print pre +"  children: "

            if element.ax.AXChildren is not None:
                c = 0
                cinv = inv+1
                ctypeMap = {}
                for child in element.ax.AXChildren:
                    UIAutomator.debug(UIElement(child, element.app), depth, c, cinv, ctypeMap)
                    c = c + 1
